@@ -1,7 +1,7 @@
 # PGP::Sign -- Create a PGP signature for data, securely.  -*- perl -*-
-# $Id: Sign.pm,v 0.8 1997/08/18 14:26:27 eagle Exp $
+# $Id: Sign.pm,v 0.9 1998/07/05 10:31:55 eagle Exp $
 #
-# Copyright 1997 by Russ Allbery <rra@stanford.edu>
+# Copyright 1997, 1998 by Russ Allbery <rra@stanford.edu>
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
@@ -31,14 +31,14 @@ use Fcntl qw(O_WRONLY O_CREAT O_EXCL);
 use FileHandle ();
 use IPC::Open3 qw(open3);
 
+use strict;
+use vars qw(@ERROR @EXPORT @EXPORT_OK @ISA $MUNGE $PGP $TMPDIR $VERSION);
+
 @ISA       = qw(Exporter);
 @EXPORT    = qw(pgp_sign pgp_verify);
 @EXPORT_OK = qw(pgp_error);
 
-use strict;
-use vars qw(@ERROR @EXPORT @EXPORT_OK @ISA $MUNGE $PGP $TMPDIR $VERSION);
-
-($VERSION = (split (' ', q$Revision: 0.8 $ ))[1]) =~ s/\.(\d)$/.0$1/;
+($VERSION = (split (' ', q$Revision: 0.9 $ ))[1]) =~ s/\.(\d)$/.0$1/;
 
 
 ############################################################################
@@ -65,8 +65,7 @@ $MUNGE = 0;
 ############################################################################
 
 # This function actually sends the data to a file handle.  It's necessary to
-# implement munging (stripping trailing spaces on a line and quoting leading
-# dashes).
+# implement munging (stripping trailing spaces on a line).
 {
     my $spaces = '';
     sub output {
@@ -180,7 +179,7 @@ sub pgp_sign {
     my $signature = new FileHandle;
     my $errors = new FileHandle;
     my @command = ($PGP, '+batchmode', '-sbaft', '-u', $keyid);
-    eval { open3 ($pgp, $signature, $errors, @command) };
+    my $pid = eval { open3 ($pgp, $signature, $errors, @command) };
     if ($@) {
         @ERROR = ($@, "Execution of pgp failed.\n");
         return undef;
@@ -193,25 +192,28 @@ sub pgp_sign {
     # All done.  Close the pipe to PGP, clean up, and see if we succeeded.
     # If not, save the error output and return undef.
     close $pgp;
-    if ($? != 0) {
-        @ERROR = (<$errors>, "PGP returned exit status $?\n");
-        return undef;
-    }
+    local $/ = "\n";
+    my @errors = <$errors>;
     my @signature = <$signature>;
     close $signature;
     close $errors;
     close $passfh;
+    waitpid ($pid, 0);
+    if ($? != 0) {
+        @ERROR = (@errors, "PGP returned exit status $?\n");
+        return undef;
+    }
 
     # Now, clean up the returned signature and return it, along with the
     # version number if desired.
     while ((shift @signature) ne "-----BEGIN PGP MESSAGE-----\n") {
-        if (not @signature) {
+        unless (@signature) {
             @ERROR = ("No signature from PGP (command not found?)\n");
             return undef;
         }
     }
     my $version;
-    while ($signature[0] ne "\n") {
+    while ($signature[0] ne "\n" && @signature) {
         ($version) = ((shift @signature) =~ /^Version:\s+(.*?)\s*$/);
     }
     shift @signature;
@@ -279,9 +281,10 @@ sub pgp_verify {
     # Check for the message that gives us the key status and return the
     # appropriate thing to our caller.
     local $_;
+    local $/ = "\n";
     my $signer;
     while (<$pgp>) {
-        if (/^Good signature from user(?::\s+(.*)|\s+\"(.*)\"\.)\n$/) {
+        if (/^Good signature from user(?::\s+(.*)|\s+\"(.*)\"\.)$/) {
             $signer = $+;
             last;
         } elsif (/^Bad signature /) {
@@ -443,15 +446,16 @@ before creating the signature.
 
 =head1 AUTHOR
 
-Russ Allbery <rra@stanford.edu>
+Russ Allbery E<lt>rra@stanford.eduE<gt>.
 
 =head1 HISTORY
 
-Based heavily on work by Andrew Gierth <andrew@erlenstar.demon.co.uk> and
-benefitting greatly from input, comments, suggestions, and help from him,
-this module came about in the process of implementing PGPMoose signatures
-and control message signatures for Usenet.  PGPMoose is the idea of Greg
-Rose <ggr@usenix.org>, and signcontrol and pgpverify are the idea of David
-Lawrence <tale@isc.org>.
+Based heavily on work by Andrew Gierth
+E<lt>andrew@erlenstar.demon.co.ukE<gt> and benefitting greatly from input,
+comments, suggestions, and help from him, this module came about in the
+process of implementing PGPMoose signatures and control message signatures
+for Usenet.  PGPMoose is the idea of Greg Rose E<lt>ggr@usenix.orgE<gt>,
+and signcontrol and pgpverify are the idea of David Lawrence
+E<lt>tale@isc.orgE<gt>.
 
 =cut
